<?php

/**
 * Isotope eCommerce for Contao Open Source CMS
 *
 * Copyright (C) 2009-2014 terminal42 gmbh & Isotope eCommerce Workgroup
 *
 * @package    Isotope
 * @link       http://isotopeecommerce.org
 * @license    http://opensource.org/licenses/lgpl-3.0.html
 */


/**
 * Class DC_TablePageId
 *
 * @copyright  Isotope eCommerce Workgroup 2009-2012
 * @author     Fred Bliss <fred.bliss@intelligentspark.com>
 * @author     Andreas Schempp <andreas.schempp@terminal42.ch>
 */
class DC_TablePageId extends \DC_Table
{

    /**
     * List all records of a particular table
     * @return string
     */
    public function showAll()
    {
        $return = '';
        $this->limit = '';
        $this->bid = 'tl_buttons';

        // Clean up old tl_undo and tl_log entries
        if ($this->strTable == 'tl_undo' && strlen($GLOBALS['TL_CONFIG']['undoPeriod']))
        {
            $this->Database->prepare("DELETE FROM tl_undo WHERE tstamp<?")
                           ->execute(intval(time() - $GLOBALS['TL_CONFIG']['undoPeriod']));
        }
        elseif ($this->strTable == 'tl_log' && strlen($GLOBALS['TL_CONFIG']['logPeriod']))
        {
            $this->Database->prepare("DELETE FROM tl_log WHERE tstamp<?")
                           ->execute(intval(time() - $GLOBALS['TL_CONFIG']['logPeriod']));
        }

        $this->reviseTable();

        // Add to clipboard
        if (\Input::get('act') == 'paste')
        {
            $arrClipboard = $this->Session->get('CLIPBOARD');

            $arrClipboard[$this->strTable] = array
            (
                'id' => \Input::get('id'),
                'childs' => \Input::get('childs'),
                'mode' => \Input::get('mode')
            );

            $this->Session->set('CLIPBOARD', $arrClipboard);
        }

        // Custom filter
        if (is_array($GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['filter']) && !empty($GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['filter']))
        {
            foreach ($GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['filter'] as $filter)
            {
                $this->procedure[] = $filter[0];
                $this->values[] = $filter[1];
            }
        }

        // Render view
        if ($this->treeView)
        {
            $return .= $this->treeView();
        }
        else
        {
            if (\Input::get('table') && $this->ptable && $this->Database->fieldExists('page_id', $this->strTable))
            {
                $this->procedure[] = 'page_id=?';
                $this->values[] = CURRENT_ID;
            }

            $return .= $this->panel();
            $return .= ($GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['mode'] == 4) ? $this->parentView() : $this->listView();

            // Add another panel at the end of the page
            if (strpos($GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['panelLayout'], 'limit') !== false && ($strLimit = $this->limitMenu(true)) != false)
            {
                $return .= '

<form action="'.ampersand(\Environment::get('request'), true).'" class="tl_form" method="post">
<div class="tl_formbody">
<input type="hidden" name="FORM_SUBMIT" value="tl_filters_limit">
<input type="hidden" name="REQUEST_TOKEN" value="'.REQUEST_TOKEN.'">

<div class="tl_panel_bottom">

<div class="tl_submit_panel tl_subpanel">
<input type="image" name="btfilter" id="btfilter" src="' . TL_FILES_URL . 'system/themes/' . $this->getTheme() . '/images/reload.gif" class="tl_img_submit" title="' . $GLOBALS['TL_LANG']['MSC']['apply'] . '" alt="' . $GLOBALS['TL_LANG']['MSC']['apply'] . '">
</div>' . $strLimit . '

<div class="clear"></div>

</div>

</div>
</form>
';
            }
        }

        // Store the current IDs
        $session = $this->Session->getData();
        $session['CURRENT']['IDS'] = $this->current;
        $this->Session->setData($session);

        return $return;
    }



    /**
     * Assign a new position to an existing record
     * @param boolean
     */
    public function cut($blnDoNotRedirect=false)
    {
        $cr = array();

        // ID and page_id are mandatory
        if (!$this->intId || !strlen(\Input::get('page_id')))
        {
            \Controller::redirect($this->getReferer());
        }

        // Get the new position
        $this->getNewPosition('cut', \Input::get('page_id'), (\Input::get('mode') == '2' ? true : false));

        // Avoid circular references when there is no parent table
        if ($this->Database->fieldExists('page_id', $this->strTable) && !strlen($this->ptable))
        {
            $cr = \Database::getInstance()->getChildRecords($this->intId, $this->strTable);
            $cr[] = $this->intId;
        }

        // Empty clipboard
        $arrClipboard = $this->Session->get('CLIPBOARD');
        $arrClipboard[$this->strTable] = array();
        $this->Session->set('CLIPBOARD', $arrClipboard);

        // Update the record
        if (in_array($this->set['page_id'], $cr))
        {
            \System::log('Attempt to relate record '.$this->intId.' of table "'.$this->strTable.'" to its child record '.\Input::get('page_id').' (circular reference)', 'DC_Table cut()', TL_ERROR);
            \Controller::redirect('contao/main.php?act=error');
        }

        $this->set['tstamp'] = time();

        // HOOK: style sheet category
        if ($this->strTable == 'tl_style')
        {
            $filter = $this->Session->get('filter');
            $category = $filter['tl_style_' . CURRENT_ID]['category'];

            if ($category != '')
            {
                $this->set['category'] = $category;
            }
        }

        $this->Database->prepare("UPDATE " . $this->strTable . " %s WHERE id=?")
                       ->set($this->set)
                       ->execute($this->intId);

        // Call the oncut_callback
        if (is_array($GLOBALS['TL_DCA'][$this->strTable]['config']['oncut_callback']))
        {
            foreach ($GLOBALS['TL_DCA'][$this->strTable]['config']['oncut_callback'] as $callback)
            {
                $this->import($callback[0]);
                $this->$callback[0]->$callback[1]($this);
            }
        }

        if (!$blnDoNotRedirect)
        {
            \Controller::redirect($this->getReferer());
        }
    }


    /**
     * Move all selected records
     */
    public function cutAll()
    {
        // page_id is mandatory
        if (!strlen(\Input::get('page_id')))
        {
            \Controller::redirect($this->getReferer());
        }

        $arrClipboard = $this->Session->get('CLIPBOARD');

        if (isset($arrClipboard[$this->strTable]) && is_array($arrClipboard[$this->strTable]['id']))
        {
            foreach ($arrClipboard[$this->strTable]['id'] as $id)
            {
                $this->intId = $id;
                $this->cut(true);
                \Input::setGet('page_id', $id);
                \Input::setGet('mode', 1);
            }
        }

        \Controller::redirect($this->getReferer());
    }


    /**
     * Calculate the new position of a moved or inserted record
     * @param string
     * @param integer
     * @param boolean
     */
    protected function getNewPosition($mode, $page_id=null, $insertInto=false)
    {
        // If there is page_id and sorting
        if ($this->Database->fieldExists('page_id', $this->strTable) && $this->Database->fieldExists('sorting', $this->strTable))
        {
            // page_id is not set - only valid for duplicated records, as they get the same parent ID as the original record!
            if ($page_id === null && $this->intId && $mode == 'copy')
            {
                $page_id = $this->intId;
            }

            // page_id is set (insert after or into the parent record)
            if (is_numeric($page_id))
            {
                // Insert the current record at the beginning when inserting into the parent record
                if ($insertInto)
                {
                    $newpage_id = $page_id;
                    $objSorting = $this->Database->prepare("SELECT MIN(sorting) AS sorting FROM " . $this->strTable . " WHERE page_id=?")
                                                 ->executeUncached($page_id);

                    // Select sorting value of the first record
                    if ($objSorting->numRows)
                    {
                        $curSorting = $objSorting->sorting;

                        // Resort if the new sorting value is not an integer or smaller than 1
                        if (($curSorting % 2) != 0 || $curSorting < 1)
                        {
                            $objNewSorting = $this->Database->prepare("SELECT id, sorting FROM " . $this->strTable . " WHERE page_id=? ORDER BY sorting" )
                                                            ->executeUncached($page_id);

                            $count = 2;
                            $newSorting = 128;

                            while ($objNewSorting->next())
                            {
                                $this->Database->prepare("UPDATE " . $this->strTable . " SET sorting=? WHERE id=?")
                                               ->limit(1)
                                               ->execute(($count++*128), $objNewSorting->id);
                            }
                        }

                        // Else new sorting = (current sorting / 2)
                        else $newSorting = ($curSorting / 2);
                    }

                    // Else new sorting = 128
                    else $newSorting = 128;
                }

                // Else insert the current record after the parent record
                elseif ($page_id > 0)
                {
                    $objSorting = $this->Database->prepare("SELECT * FROM " . $this->strTable . " WHERE id=?")
                                                 ->limit(1)
                                                 ->executeUncached($page_id);

                    // Set parent ID of the current record as new parent ID
                    if ($objSorting->numRows)
                    {
                        $newpage_id = $objSorting->page_id;
                        $curSorting = $objSorting->sorting;

                        // Do not proceed without a parent ID
                        if (is_numeric($newpage_id))
                        {
                            $objNextSorting = $this->Database->prepare("SELECT MIN(sorting) AS sorting FROM " . $this->strTable . " WHERE page_id=? AND sorting>?")
                                                               ->executeUncached($newpage_id, $curSorting);

                            // Select sorting value of the next record
                            if ($objNextSorting->sorting !== null)
                            {
                                $nxtSorting = $objNextSorting->sorting;

                                // Resort if the new sorting value is no integer or bigger than a MySQL integer
                                if ((($curSorting + $nxtSorting) % 2) != 0 || $nxtSorting >= 4294967295)
                                {
                                    $count = 1;

                                    $objNewSorting = $this->Database->prepare("SELECT id, sorting FROM " . $this->strTable . " WHERE page_id=? ORDER BY sorting")
                                                                    ->executeUncached($newpage_id);

                                    while ($objNewSorting->next())
                                    {
                                        $this->Database->prepare("UPDATE " . $this->strTable . " SET sorting=? WHERE id=?")
                                                       ->execute(($count++*128), $objNewSorting->id);

                                        if ($objNewSorting->sorting == $curSorting)
                                        {
                                            $newSorting = ($count++*128);
                                        }
                                    }
                                }

                                // Else new sorting = (current sorting + next sorting) / 2
                                else $newSorting = (($curSorting + $nxtSorting) / 2);
                            }

                            // Else new sorting = (current sorting + 128)
                            else $newSorting = ($curSorting + 128);
                        }
                    }

                    // Use the given parent ID as parent ID
                    else
                    {
                        $newpage_id = $page_id;
                        $newSorting = 128;
                    }
                }

                // Set new sorting and new parent ID
                $this->set['page_id'] = intval($newpage_id);
                $this->set['sorting'] = intval($newSorting);
            }
        }

        // If there is only page_id
        elseif ($this->Database->fieldExists('page_id', $this->strTable))
        {
            // page_id is not set - only valid for duplicated records, as they get the same parent ID as the original record!
            if ($page_id === null && $this->intId && $mode == 'copy')
            {
                $page_id = $this->intId;
            }

            // page_id is set (insert after or into the parent record)
            if (is_numeric($page_id))
            {
                // Insert the current record into the parent record
                if ($insertInto)
                {
                    $this->set['page_id'] = $page_id;
                }

                // Else insert the current record after the parent record
                elseif ($page_id > 0)
                {
                    $objParentRecord = $this->Database->prepare("SELECT * FROM " . $this->strTable . " WHERE id=?")
                                                      ->limit(1)
                                                      ->executeUncached($page_id);

                    if ($objParentRecord->numRows)
                    {
                        $this->set['page_id'] = $objParentRecord->page_id;
                    }
                }
            }
        }

        // If there is only sorting
        elseif ($this->Database->fieldExists('sorting', $this->strTable))
        {
            // ID is set (insert after the current record)
            if ($this->intId)
            {
                $objCurrentRecord = $this->Database->prepare("SELECT * FROM " . $this->strTable . " WHERE id=?")
                                                   ->limit(1)
                                                    ->executeUncached($this->intId);

                // Select current record
                if ($objCurrentRecord->numRows)
                {
                    $curSorting = $objCurrentRecord->sorting;

                    $objNextSorting = $this->Database->prepare("SELECT MIN(sorting) AS sorting FROM " . $this->strTable . " WHERE sorting>?")
                                                     ->executeUncached($curSorting);

                    // Select sorting value of the next record
                    if ($objNextSorting->numRows)
                    {
                        $nxtSorting = $objNextSorting->sorting;

                        // Resort if the new sorting value is no integer or bigger than a MySQL integer field
                        if ((($curSorting + $nxtSorting) % 2) != 0 || $nxtSorting >= 4294967295)
                        {
                            $count = 1;

                            $objNewSorting = $this->Database->executeUncached("SELECT id, sorting FROM " . $this->strTable . " ORDER BY sorting");

                            while ($objNewSorting->next())
                            {
                                $this->Database->prepare("UPDATE " . $this->strTable . " SET sorting=? WHERE id=?")
                                               ->execute(($count++*128), $objNewSorting->id);

                                if ($objNewSorting->sorting == $curSorting)
                                {
                                    $newSorting = ($count++*128);
                                }
                            }
                        }

                        // Else new sorting = (current sorting + next sorting) / 2
                        else $newSorting = (($curSorting + $nxtSorting) / 2);
                    }

                    // Else new sorting = (current sorting + 128)
                    else $newSorting = ($curSorting + 128);

                    // Set new sorting
                    $this->set['sorting'] = intval($newSorting);
                }

                // ID is not set (insert at the end)
                else
                {
                    $objNextSorting = $this->Database->executeUncached("SELECT MAX(sorting) AS sorting FROM " . $this->strTable);

                    if ($objNextSorting->numRows)
                    {
                        $this->set['sorting'] = intval($objNextSorting->sorting + 128);
                    }
                }
            }
        }
    }


    /**
     * Change the order of two neighbour database records
     */
    public function move()
    {
        // Proceed only if all mandatory variables are set
        if ($this->intId && \Input::get('sid') && (!$GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['root'] || !in_array($this->intId, $this->root)))
        {
            $objRow = $this->Database->prepare("SELECT * FROM " . $this->strTable . " WHERE id=? OR id=?")
                                     ->limit(2)
                                     ->execute($this->intId, \Input::get('sid'));

            $row = $objRow->fetchAllAssoc();

            if ($row[0]['page_id'] == $row[1]['page_id'])
            {
                $this->Database->prepare("UPDATE " . $this->strTable . " SET sorting=? WHERE id=?")
                               ->execute($row[0]['sorting'], $row[1]['id']);

                $this->Database->prepare("UPDATE " . $this->strTable . " SET sorting=? WHERE id=?")
                               ->execute($row[1]['sorting'], $row[0]['id']);
            }
        }

        \Controller::redirect($this->getReferer());
    }


    /**
     * Delete all incomplete and unrelated records
     */
    protected function reviseTable()
    {
        $reload = false;
        $ptable = $GLOBALS['TL_DCA'][$this->strTable]['config']['ptable'];
        $ctable = $GLOBALS['TL_DCA'][$this->strTable]['config']['ctable'];

        $new_records = $this->Session->get('new_records');

        // HOOK: add custom logic
        if (isset($GLOBALS['TL_HOOKS']['reviseTable']) && is_array($GLOBALS['TL_HOOKS']['reviseTable']))
        {
            foreach ($GLOBALS['TL_HOOKS']['reviseTable'] as $callback)
            {
                $this->import($callback[0]);
                $status = $this->$callback[0]->$callback[1]($this->strTable, $new_records[$this->strTable], $ptable, $ctable);

                if ($status === true)
                {
                    $reload = true;
                }
            }
        }

        // Delete all new but incomplete records (tstamp=0)
        if (is_array($new_records[$this->strTable]) && !empty($new_records[$this->strTable]))
        {
            $objStmt = $this->Database->execute("DELETE FROM " . $this->strTable . " WHERE id IN(" . implode(',', array_map('intval', $new_records[$this->strTable])) . ") AND tstamp=0");

            if ($objStmt->affectedRows > 0)
            {
                $reload = true;
            }
        }

        // Delete all records of the current table that are not related to the parent table
        if (strlen($ptable))
        {
            $objStmt = $this->Database->execute("DELETE FROM " . $this->strTable . " WHERE NOT EXISTS (SELECT * FROM " . $ptable . " WHERE " . $this->strTable . ".page_id = " . $ptable . ".id)");

            if ($objStmt->affectedRows > 0)
            {
                $reload = true;
            }
        }

        // Delete all records of the child table that are not related to the current table
        if (is_array($ctable) && !empty($ctable))
        {
            foreach ($ctable as $v)
            {
                if (strlen($v))
                {
                    $objStmt = $this->Database->execute("DELETE FROM " . $v . " WHERE NOT EXISTS (SELECT * FROM " . $this->strTable . " WHERE " . $v . ".page_id = " . $this->strTable . ".id)");

                    if ($objStmt->affectedRows > 0)
                    {
                        $reload = true;
                    }
                }
            }
        }

        // Reload the page
        if ($reload)
        {
            $this->reload();
        }
    }


    /**
      * Show header of the parent table and list all records of the current table
     * @return string
     */
    protected function parentView()
    {
        $blnClipboard = false;
        $arrClipboard = $this->Session->get('CLIPBOARD');
        $table = ($GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['mode'] == 6) ? $this->ptable : $this->strTable;
        $blnHasSorting = $GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['fields'][0] == 'sorting';
        $blnMultiboard = false;

        // Check clipboard
        if (isset($arrClipboard[$table]) && !empty($arrClipboard[$table]))
        {
            $blnClipboard = true;
            $arrClipboard = $arrClipboard[$table];

            if (is_array($arrClipboard['id']))
            {
                $blnMultiboard = true;
            }
        }

        // Load language file and data container array of the parent table
        \System::loadLanguageFile($this->ptable);
        $this->loadDataContainer($this->ptable);

        $return = '
<div id="tl_buttons">
<a href="'.$this->getReferer(true, $this->ptable).'" class="header_back" title="'.specialchars($GLOBALS['TL_LANG']['MSC']['backBT']).'" accesskey="b" onclick="Backend.getScrollOffset();">'.$GLOBALS['TL_LANG']['MSC']['backBT'].'</a>' . ((\Input::get('act') != 'select') ? ' &#160; :: &#160; ' . (!$GLOBALS['TL_DCA'][$this->strTable]['config']['closed'] ? '
<a href="'.$this->addToUrl(($blnHasSorting ? 'act=paste&amp;mode=create' : 'act=create&amp;mode=2&amp;page_id='.$this->intId)).'" class="header_new" title="'.specialchars($GLOBALS['TL_LANG'][$this->strTable]['new'][1]).'" accesskey="n" onclick="Backend.getScrollOffset();">'.$GLOBALS['TL_LANG'][$this->strTable]['new'][0].'</a>' : '') . $this->generateGlobalButtons() . ($blnClipboard ? ' &nbsp; :: &nbsp; <a href="'.$this->addToUrl('clipboard=1').'" class="header_clipboard" title="'.specialchars($GLOBALS['TL_LANG']['MSC']['clearClipboard']).'" accesskey="x">'.$GLOBALS['TL_LANG']['MSC']['clearClipboard'].'</a>' : '') : '') . '
</div>' . $this->getMessages(true);

        // Get all details of the parent record
        $objParent = $this->Database->prepare("SELECT * FROM " . $this->ptable . " WHERE id=?")
                                    ->limit(1)
                                    ->execute(CURRENT_ID);

        if ($objParent->numRows < 1)
        {
            return $return;
        }

        $return .= ((\Input::get('act') == 'select') ? '

<form action="'.ampersand(\Environment::get('request'), true).'" id="tl_select" class="tl_form" method="post">
<div class="tl_formbody">
<input type="hidden" name="FORM_SUBMIT" value="tl_select">
<input type="hidden" name="REQUEST_TOKEN" value="'.REQUEST_TOKEN.'">' : '').'

<div class="tl_listing_container parent_view">

<div class="tl_header" onmouseover="Theme.hoverDiv(this, 1);" onmouseout="Theme.hoverDiv(this, 0);">';

        // List all records of the child table
        if (!\Input::get('act') || \Input::get('act') == 'paste' || \Input::get('act') == 'select')
        {
            // Header
            $imagePasteNew = \Image::getHtml('new.gif', $GLOBALS['TL_LANG'][$this->strTable]['pasteafter'][0]);
            $imagePasteAfter = \Image::getHtml('pasteafter.gif', $GLOBALS['TL_LANG'][$this->strTable]['pasteafter'][0], 'class="blink"');
            $imageEditHeader = \Image::getHtml('edit.gif', $GLOBALS['TL_LANG'][$this->strTable]['editheader'][0]);

            $return .= '
<div class="tl_content_right">'.((\Input::get('act') == 'select') ? '
<label for="tl_select_trigger" class="tl_select_label">'.$GLOBALS['TL_LANG']['MSC']['selectAll'].'</label> <input type="checkbox" id="tl_select_trigger" onclick="Backend.toggleCheckboxes(this)" class="tl_tree_checkbox">' : (!$GLOBALS['TL_DCA'][$this->ptable]['config']['notEditable'] ? '
<a href="'.preg_replace('/&(amp;)?table=[^& ]*/i', (strlen($this->ptable) ? '&amp;table='.$this->ptable : ''), $this->addToUrl('act=edit')).'" title="'.specialchars($GLOBALS['TL_LANG'][$this->strTable]['editheader'][1]).'">'.$imageEditHeader.'</a>' : '') . (($blnHasSorting && !$GLOBALS['TL_DCA'][$this->strTable]['config']['closed']) ? ' <a href="'.$this->addToUrl('act=create&amp;mode=2&amp;page_id='.$objParent->id.'&amp;id='.$this->intId).'" title="'.specialchars($GLOBALS['TL_LANG'][$this->strTable]['pastenew'][0]).'">'.$imagePasteNew.'</a>' : '') . ($blnClipboard ? ' <a href="'.$this->addToUrl('act='.$arrClipboard['mode'].'&amp;mode=2&amp;page_id='.$objParent->id . (!$blnMultiboard ? '&amp;id='.$arrClipboard['id'] : '')).'" title="'.specialchars($GLOBALS['TL_LANG'][$this->strTable]['pasteafter'][0]).'" onclick="Backend.getScrollOffset();">'.$imagePasteAfter.'</a>' : '')) . '
</div>';

            // Format header fields
            $add = array();
            $headerFields = $GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['headerFields'];

            foreach ($headerFields as $v)
            {
                $_v = deserialize($objParent->$v);

                if (is_array($_v))
                {
                    $_v = implode(', ', $_v);
                }
                elseif ($GLOBALS['TL_DCA'][$this->ptable]['fields'][$v]['inputType'] == 'checkbox' && !$GLOBALS['TL_DCA'][$this->ptable]['fields'][$v]['eval']['multiple'])
                {
                    $_v = strlen($_v) ? $GLOBALS['TL_LANG']['MSC']['yes'] : $GLOBALS['TL_LANG']['MSC']['no'];
                }
                elseif ($_v && $GLOBALS['TL_DCA'][$this->ptable]['fields'][$v]['eval']['rgxp'] == 'date')
                {
                    $_v = \System::parseDate($GLOBALS['TL_CONFIG']['dateFormat'], $_v);
                }
                elseif ($_v && $GLOBALS['TL_DCA'][$this->ptable]['fields'][$v]['eval']['rgxp'] == 'time')
                {
                    $_v = \System::parseDate($GLOBALS['TL_CONFIG']['timeFormat'], $_v);
                }
                elseif ($_v && $GLOBALS['TL_DCA'][$this->ptable]['fields'][$v]['eval']['rgxp'] == 'datim')
                {
                    $_v = \System::parseDate($GLOBALS['TL_CONFIG']['datimFormat'], $_v);
                }
                elseif ($v == 'tstamp')
                {
                    $objMaxTstamp = $this->Database->prepare("SELECT MAX(tstamp) AS tstamp FROM " . $this->strTable . " WHERE page_id=?")
                                                   ->execute($objParent->id);

                    if (!$objMaxTstamp->tstamp)
                    {
                        $objMaxTstamp->tstamp = $objParent->tstamp;
                    }

                    $_v = \System::parseDate($GLOBALS['TL_CONFIG']['datimFormat'], max($objParent->tstamp, $objMaxTstamp->tstamp));
                }
                elseif (isset($GLOBALS['TL_DCA'][$this->ptable]['fields'][$v]['foreignKey']))
                {
                    $arrForeignKey = explode('.', $GLOBALS['TL_DCA'][$this->ptable]['fields'][$v]['foreignKey'], 2);

                    $objLabel = $this->Database->prepare("SELECT " . $arrForeignKey[1] . " AS value FROM " . $arrForeignKey[0] . " WHERE id=?")
                                               ->limit(1)
                                               ->execute($_v);

                    if ($objLabel->numRows)
                    {
                        $_v = $objLabel->value;
                    }
                }
                elseif (is_array($GLOBALS['TL_DCA'][$this->ptable]['fields'][$v]['reference'][$_v]))
                {
                    $_v = $GLOBALS['TL_DCA'][$this->ptable]['fields'][$v]['reference'][$_v][0];
                }
                elseif (isset($GLOBALS['TL_DCA'][$this->ptable]['fields'][$v]['reference'][$_v]))
                {
                    $_v = $GLOBALS['TL_DCA'][$this->ptable]['fields'][$v]['reference'][$_v];
                }
                elseif (array_is_assoc($GLOBALS['TL_DCA'][$this->ptable]['fields'][$v]['options']))
                {
                    $_v = $GLOBALS['TL_DCA'][$this->ptable]['fields'][$v]['options'][$_v];
                }

                // Add sorting field
                if (strlen($_v))
                {
                    $key = strlen($GLOBALS['TL_LANG'][$this->ptable][$v][0]) ? $GLOBALS['TL_LANG'][$this->ptable][$v][0]  : $v;
                    $add[$key] = $_v;
                }
            }

            // Output header data
            $return .= '

<table class="tl_header_table">';

            foreach ($add as $k=>$v)
            {
                if (is_array($v))
                {
                    $v = $v[0];
                }

                $return .= '
  <tr>
    <td><span class="tl_label">'.$k.':</span> </td>
    <td>'.$v.'</td>
  </tr>';
            }

            $return .= '
</table>
</div>';

            $orderBy = array();
            $firstOrderBy = array();

            // Add all records of the current table
            $query = "SELECT * FROM " . $this->strTable;

            if (is_array($this->orderBy) && strlen($this->orderBy[0]))
            {
                $orderBy = $this->orderBy;
                $firstOrderBy = preg_replace('/\s+.*$/i', '', $orderBy[0]);

                // Order by the foreign key
                if (isset($GLOBALS['TL_DCA'][$this->strTable]['fields'][$firstOrderBy]['foreignKey']))
                {
                    $key = explode('.', $GLOBALS['TL_DCA'][$this->strTable]['fields'][$firstOrderBy]['foreignKey'], 2);
                    $query = "SELECT *, (SELECT ". $key[1] ." FROM ". $key[0] ." WHERE ". $this->strTable .".". $firstOrderBy ."=". $key[0] .".id) AS foreignKey FROM " . $this->strTable;
                    $orderBy[0] = 'foreignKey';
                }
            }
            elseif (is_array($GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['fields']))
            {
                $orderBy = $GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['fields'];
                $firstOrderBy = preg_replace('/\s+.*$/i', '', $orderBy[0]);
            }

            if (!empty($this->procedure))
            {
                $query .= " WHERE " . implode(' AND ', $this->procedure);
            }

            if (is_array($this->root) && count($this->root) > 0) // Can't use empty() because its an object property (using __get)
            {
                $query .= (count($this->procedure) ? " AND " : " WHERE ") . "id IN(" . implode(',', array_map('intval', $this->root)) . ")";
            }

            if (is_array($orderBy) && !empty($orderBy))
            {
                $query .= " ORDER BY " . implode(', ', $orderBy);
            }

            $objOrderByStmt = $this->Database->prepare($query);

            if (strlen($this->limit))
            {
                $arrLimit = explode(',', $this->limit);
                $objOrderByStmt->limit($arrLimit[1], $arrLimit[0]);
            }

            $objOrderBy = $objOrderByStmt->execute($this->values);

            if ($objOrderBy->numRows < 1)
            {
                return $return . '
<p class="tl_empty_parent_view">'.$GLOBALS['TL_LANG']['MSC']['noResult'].'</p>

</div>';
            }

            if (is_array($GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['child_record_callback']))
            {
                $strClass = $GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['child_record_callback'][0];
                $strMethod = $GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['child_record_callback'][1];

                $this->import($strClass);
                $row = $objOrderBy->fetchAllAssoc();
                $strGroup = '';

                // Make items sortable
                if ($blnHasSorting)
                {
                    $return .= '

<ul id="ul_' . CURRENT_ID . '" class="sortable">';
                }

                for ($i=0, $count=count($row); $i<$count; $i++)
                {
                    $this->current[] = $row[$i]['id'];
                    $imagePasteAfter = \Image::getHtml('pasteafter.gif', sprintf($GLOBALS['TL_LANG'][$this->strTable]['pasteafter'][1], $row[$i]['id']), 'class="blink"');
                    $imagePasteNew = \Image::getHtml('new.gif', sprintf($GLOBALS['TL_LANG'][$this->strTable]['pastenew'][1], $row[$i]['id']));

                    // Decrypt encrypted value
                    foreach ($row[$i] as $k=>$v)
                    {
                        if ($GLOBALS['TL_DCA'][$table]['fields'][$k]['eval']['encrypt'])
                        {
                            $v = deserialize($v);

                            $this->import('Encryption');
                            $row[$i][$k] = $this->Encryption->decrypt($v);
                        }
                    }

                    // Make items sortable
                    if ($blnHasSorting)
                    {
                        $return .= '
<li id="li_' . $row[$i]['id'] . '">';
                    }

                    // Add the group header
                    if (!$GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['disableGrouping'] && $firstOrderBy != 'sorting')
                    {
                        $sortingMode = (count($orderBy) == 1 && $firstOrderBy == $orderBy[0] && strlen($GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['flag']) && !strlen($GLOBALS['TL_DCA'][$this->strTable]['fields'][$firstOrderBy]['flag'])) ? $GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['flag'] : $GLOBALS['TL_DCA'][$this->strTable]['fields'][$firstOrderBy]['flag'];
                        $remoteNew = $this->formatCurrentValue($firstOrderBy, $row[$i][$firstOrderBy], $sortingMode);
                        $group = $this->formatGroupHeader($firstOrderBy, $remoteNew, $sortingMode, $row);

                        if ($group != $strGroup)
                        {
                            $return .= "\n\n" . '<div class="tl_content_header">'.$group.'</div>';
                            $strGroup = $group;
                        }
                    }

                    $return .= '

<div class="tl_content block'.(($GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['child_record_class'] != '') ? ' ' . $GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['child_record_class'] : '').'" onmouseover="Theme.hoverDiv(this, 1);" onmouseout="Theme.hoverDiv(this, 0);">
<div class="tl_content_right">';

                    // Edit multiple
                    if (\Input::get('act') == 'select')
                    {
                        $return .= '<input type="checkbox" name="IDS[]" id="ids_'.$row[$i]['id'].'" class="tl_tree_checkbox" value="'.$row[$i]['id'].'">';
                    }

                    // Regular buttons
                    else
                    {
                        $return .= $this->generateButtons($row[$i], $this->strTable, $this->root, false, null, $row[($i-1)]['id'], $row[($i+1)]['id']);

                        // Sortable table
                        if ($blnHasSorting)
                        {
                            // Create new button
                            if (!$GLOBALS['TL_DCA'][$this->strTable]['config']['closed'])
                            {
                                $return .= ' <a href="'.$this->addToUrl('act=create&amp;mode=1&amp;page_id='.$row[$i]['id'].'&amp;id='.$objParent->id).'" title="'.specialchars(sprintf($GLOBALS['TL_LANG'][$this->strTable]['pastenew'][1], $row[$i]['id'])).'">'.$imagePasteNew.'</a>';
                            }

                            // Prevent circular references
                            if ($blnClipboard && $arrClipboard['mode'] == 'cut' && $row[$i]['id'] == $arrClipboard['id'] || $blnMultiboard && $arrClipboard['mode'] == 'cutAll' && in_array($row[$i]['id'], $arrClipboard['id']))
                            {
                                $return .= ' ' . \Image::getHtml('pasteafter_.gif', '', 'class="blink"');
                            }

                            // Copy/move multiple
                            elseif ($blnMultiboard)
                            {
                                $return .= ' <a href="'.$this->addToUrl('act='.$arrClipboard['mode'].'&amp;mode=1&amp;page_id='.$row[$i]['id']).'" title="'.specialchars(sprintf($GLOBALS['TL_LANG'][$this->strTable]['pasteafter'][1], $row[$i]['id'])).'" onclick="Backend.getScrollOffset();">'.$imagePasteAfter.'</a>';
                            }

                            // Paste buttons
                            elseif ($blnClipboard)
                            {
                                $return .= ' <a href="'.$this->addToUrl('act='.$arrClipboard['mode'].'&amp;mode=1&amp;page_id='.$row[$i]['id'].'&amp;id='.$arrClipboard['id']).'" title="'.specialchars(sprintf($GLOBALS['TL_LANG'][$this->strTable]['pasteafter'][1], $row[$i]['id'])).'" onclick="Backend.getScrollOffset();">'.$imagePasteAfter.'</a>';
                            }
                        }
                    }

                    $return .= '</div>'.$this->$strClass->$strMethod($row[$i]).'</div>';

                    // Make items sortable
                    if ($blnHasSorting)
                    {
                        $return .= '

</li>';
                    }
                }
            }
        }

        // Make items sortable
        if ($blnHasSorting)
        {
            $return .= '
</ul>

<script>
Isotope.makePageViewSortable("ul_' . CURRENT_ID . '");
</script>';
        }

        $return .= '

</div>';

        // Close form
        if (\Input::get('act') == 'select')
        {
            $return .= '

<div class="tl_formbody_submit" style="text-align:right;">

<div class="tl_submit_container">
  <input type="submit" name="cut" id="cut" class="tl_submit" alt="move selected records" accesskey="x" value="'.specialchars($GLOBALS['TL_LANG']['MSC']['moveSelected']).'">
</div>

</div>
</div>
</form>';
        }

        return $return;
    }


    /**
     * List all records of the current table and return them as HTML string
     * @return string
     */
    protected function listView()
    {
        $return = '';
        $table = ($GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['mode'] == 6) ? $this->ptable : $this->strTable;
        $orderBy = $GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['fields'];
        $firstOrderBy = preg_replace('/\s+.*$/i', '', $orderBy[0]);

        if (is_array($this->orderBy) && strlen($this->orderBy[0]))
        {
            $orderBy = $this->orderBy;
            $firstOrderBy = $this->firstOrderBy;
        }

        // Show only own undo steps
        if ($this->strTable == 'tl_undo')
        {
            $this->import('BackendUser', 'User');

            if (!$this->User->isAdmin)
            {
                $this->procedure[] = 'page_id=?';
                $this->values[] = $this->User->id;
            }
        }

        $query = "SELECT * FROM " . $this->strTable;

        if (count($this->procedure))
        {
            $query .= " WHERE " . implode(' AND ', $this->procedure);
        }

        if (is_array($this->root) && count($this->root)) // Can't use empty() because its an object property (using __get)
        {
            $query .= (count($this->procedure) ? " AND " : " WHERE ") . "id IN(" . implode(',', array_map('intval', $this->root)) . ")";
        }

        if (is_array($orderBy) && strlen($orderBy[0]))
        {
            foreach ($orderBy as $k=>$v)
            {
                if ($GLOBALS['TL_DCA'][$this->strTable]['fields'][$v]['eval']['findInSet'])
                {
                    if (is_array($GLOBALS['TL_DCA'][$this->strTable]['fields'][$v]['options_callback']))
                    {
                        $strClass = $GLOBALS['TL_DCA'][$this->strTable]['fields'][$v]['options_callback'][0];
                        $strMethod = $GLOBALS['TL_DCA'][$this->strTable]['fields'][$v]['options_callback'][1];

                        $this->import($strClass);
                        $keys = $this->$strClass->$strMethod($this);
                    }
                    else
                    {
                        $keys = $GLOBALS['TL_DCA'][$this->strTable]['fields'][$v]['options'];
                    }

                    if (array_is_assoc($keys))
                    {
                        $keys = array_keys($keys);
                    }

                    $orderBy[$k] = $this->Database->findInSet($v, $keys);
                }
            }

            $query .= " ORDER BY " . implode(', ', $orderBy);
        }

        if ($GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['mode'] == 1 && ($GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['flag'] % 2) == 0)
        {
            $query .= " DESC";
        }

        $objRowStmt = $this->Database->prepare($query);

        if (strlen($this->limit))
        {
            $arrLimit = explode(',', $this->limit);
            $objRowStmt->limit($arrLimit[1], $arrLimit[0]);
        }

        $objRow = $objRowStmt->execute($this->values);
        $this->bid = strlen($return) ? $this->bid : 'tl_buttons';

        // Display buttos
        if (!$GLOBALS['TL_DCA'][$this->strTable]['config']['closed'] || !empty($GLOBALS['TL_DCA'][$this->strTable]['list']['global_operations']))
        {
            $return .= '

<div id="'.$this->bid.'">'.((\Input::get('act') == 'select' || $this->ptable) ? '
<a href="'.$this->getReferer(true, $this->ptable).'" class="header_back" title="'.specialchars($GLOBALS['TL_LANG']['MSC']['backBT']).'" accesskey="b" onclick="Backend.getScrollOffset();">'.$GLOBALS['TL_LANG']['MSC']['backBT'].'</a>' : '') . (($this->ptable && \Input::get('act') != 'select') ? ' &nbsp; :: &nbsp;' : '') . ((\Input::get('act') != 'select') ? '
'.(!$GLOBALS['TL_DCA'][$this->strTable]['config']['closed'] ? '<a href="'.(strlen($this->ptable) ? $this->addToUrl('act=create' . (($GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['mode'] < 4) ? '&amp;mode=2' : '') . '&amp;page_id=' . $this->intId) : $this->addToUrl('act=create')).'" class="header_new" title="'.specialchars($GLOBALS['TL_LANG'][$this->strTable]['new'][1]).'" accesskey="n" onclick="Backend.getScrollOffset();">'.$GLOBALS['TL_LANG'][$this->strTable]['new'][0].'</a>' : '') . $this->generateGlobalButtons() : '') . '
</div>' . $this->getMessages(true);
        }

        // Return "no records found" message
        if ($objRow->numRows < 1)
        {
            $return .= '
<p class="tl_empty">'.$GLOBALS['TL_LANG']['MSC']['noResult'].'</p>';
        }

        // List records
        else
        {
            $result = $objRow->fetchAllAssoc();
            $return .= ((\Input::get('act') == 'select') ? '

<form action="'.ampersand(\Environment::get('request'), true).'" id="tl_select" class="tl_form" method="post">
<div class="tl_formbody">
<input type="hidden" name="FORM_SUBMIT" value="tl_select">
<input type="hidden" name="REQUEST_TOKEN" value="'.REQUEST_TOKEN.'">' : '').'

<div class="tl_listing_container list_view">'.((\Input::get('act') == 'select') ? '

<div class="tl_select_trigger">
<label for="tl_select_trigger" class="tl_select_label">'.$GLOBALS['TL_LANG']['MSC']['selectAll'].'</label> <input type="checkbox" id="tl_select_trigger" onclick="Backend.toggleCheckboxes(this)" class="tl_tree_checkbox">
</div>' : '').'

<table class="tl_listing">';

            // Rename each page_id to its label and resort the result (sort by parent table)
            if ($GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['mode'] == 3 && $this->Database->fieldExists('page_id', $this->strTable))
            {
                $firstOrderBy = 'page_id';
                $showFields = $GLOBALS['TL_DCA'][$table]['list']['label']['fields'];

                foreach ($result as $k=>$v)
                {
                    $objField = $this->Database->prepare("SELECT " . $showFields[0] . " FROM " . $this->ptable . " WHERE id=?")
                                               ->limit(1)
                                               ->execute($v['page_id']);

                    $result[$k]['page_id'] = $objField->$showFields[0];
                }

                $aux = array();

                foreach ($result as $row)
                {
                    $aux[] = $row['page_id'];
                }

                array_multisort($aux, SORT_ASC, $result);
            }

            // Process result and add label and buttons
            $remoteCur = false;
            $groupclass = 'tl_folder_tlist';

            foreach ($result as $row)
            {
                $args = array();
                $this->current[] = $row['id'];
                $showFields = $GLOBALS['TL_DCA'][$table]['list']['label']['fields'];

                // Label
                foreach ($showFields as $k=>$v)
                {
                    // Decrypt the value
                    if ($GLOBALS['TL_DCA'][$this->strTable]['fields'][$v]['eval']['encrypt'])
                    {
                        $row[$v] = deserialize($row[$v]);

                        $this->import('Encryption');
                        $row[$v] = $this->Encryption->decrypt($row[$v]);
                    }

                    if (strpos($v, ':') !== false)
                    {
                        list($strKey, $strTable) = explode(':', $v);
                        list($strTable, $strField) = explode('.', $strTable);

                        $objRef = $this->Database->prepare("SELECT " . $strField . " FROM " . $strTable . " WHERE id=?")
                                                 ->limit(1)
                                                 ->execute($row[$strKey]);

                        $args[$k] = $objRef->numRows ? $objRef->$strField : '';
                    }
                    elseif (in_array($GLOBALS['TL_DCA'][$this->strTable]['fields'][$v]['flag'], array(5, 6, 7, 8, 9, 10)))
                    {
                        if ($GLOBALS['TL_DCA'][$this->strTable]['fields'][$v]['eval']['rgxp'] == 'date')
                        {
                            $args[$k] = \System::parseDate($GLOBALS['TL_CONFIG']['dateFormat'], $row[$v]);
                        }
                        elseif ($GLOBALS['TL_DCA'][$this->strTable]['fields'][$v]['eval']['rgxp'] == 'time')
                        {
                            $args[$k] = \System::parseDate($GLOBALS['TL_CONFIG']['timeFormat'], $row[$v]);
                        }
                        else
                        {
                            $args[$k] = \System::parseDate($GLOBALS['TL_CONFIG']['datimFormat'], $row[$v]);
                        }
                    }
                    elseif ($GLOBALS['TL_DCA'][$this->strTable]['fields'][$v]['inputType'] == 'checkbox' && !$GLOBALS['TL_DCA'][$this->strTable]['fields'][$v]['eval']['multiple'])
                    {
                        $args[$k] = strlen($row[$v]) ? $GLOBALS['TL_DCA'][$table]['fields'][$v]['label'][0] : '';
                    }
                    else
                    {
                        $row_v = deserialize($row[$v]);

                        if (is_array($row_v))
                        {
                            $args_k = array();

                            foreach ($row_v as $option)
                            {
                                $args_k[] = strlen($GLOBALS['TL_DCA'][$table]['fields'][$v]['reference'][$option]) ? $GLOBALS['TL_DCA'][$table]['fields'][$v]['reference'][$option] : $option;
                            }

                            $args[$k] = implode(', ', $args_k);
                        }
                        elseif (isset($GLOBALS['TL_DCA'][$table]['fields'][$v]['reference'][$row[$v]]))
                        {
                            $args[$k] = is_array($GLOBALS['TL_DCA'][$table]['fields'][$v]['reference'][$row[$v]]) ? $GLOBALS['TL_DCA'][$table]['fields'][$v]['reference'][$row[$v]][0] : $GLOBALS['TL_DCA'][$table]['fields'][$v]['reference'][$row[$v]];
                        }
                        elseif (array_is_assoc($GLOBALS['TL_DCA'][$table]['fields'][$v]['options']) && isset($GLOBALS['TL_DCA'][$table]['fields'][$v]['options'][$row[$v]]))
                        {
                            $args[$k] = $GLOBALS['TL_DCA'][$table]['fields'][$v]['options'][$row[$v]];
                        }
                        else
                        {
                            $args[$k] = $row[$v];
                        }
                    }
                }

                // Shorten the label it if it is too long
                $label = vsprintf((strlen($GLOBALS['TL_DCA'][$this->strTable]['list']['label']['format']) ? $GLOBALS['TL_DCA'][$this->strTable]['list']['label']['format'] : '%s'), $args);

                if ($GLOBALS['TL_DCA'][$this->strTable]['list']['label']['maxCharacters'] > 0 && $GLOBALS['TL_DCA'][$this->strTable]['list']['label']['maxCharacters'] < strlen(strip_tags($label)))
                {
                    $label = trim(\String::substrHtml($label, $GLOBALS['TL_DCA'][$this->strTable]['list']['label']['maxCharacters'])) . ' …';
                }

                // Remove empty brackets (), [], {}, <> and empty tags from the label
                $label = preg_replace('/\( *\) ?|\[ *\] ?|\{ *\} ?|< *> ?/i', '', $label);
                $label = preg_replace('/<[^>]+>\s*<\/[^>]+>/i', '', $label);

                // Build sorting groups
                if ($GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['mode'] > 0)
                {
                    $current = $row[$firstOrderBy];
                    $orderBy = $GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['fields'];
                    $sortingMode = (count($orderBy) == 1 && $firstOrderBy == $orderBy[0] && strlen($GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['flag']) && !strlen($GLOBALS['TL_DCA'][$this->strTable]['fields'][$firstOrderBy]['flag'])) ? $GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['flag'] : $GLOBALS['TL_DCA'][$this->strTable]['fields'][$firstOrderBy]['flag'];
                    $remoteNew = $this->formatCurrentValue($firstOrderBy, $current, $sortingMode);

                    // Add the group header
                    if (!$GLOBALS['TL_DCA'][$this->strTable]['list']['sorting']['disableGrouping'] && ($remoteNew != $remoteCur || $remoteCur === false))
                    {
                        $group = $this->formatGroupHeader($firstOrderBy, $remoteNew, $sortingMode, $row);
                        $remoteCur = $remoteNew;

                        $return .= '
  <tr>
    <td colspan="2" class="'.$groupclass.'">'.$group.'</td>
  </tr>';
                        $groupclass = 'tl_folder_list';
                    }
                }

                $return .= '
  <tr onmouseover="Theme.hoverRow(this, 1);" onmouseout="Theme.hoverRow(this, 0);">
    <td class="tl_file_list">';

                // Call label callback ($row, $label, $this)
                if (is_array($GLOBALS['TL_DCA'][$this->strTable]['list']['label']['label_callback']))
                {
                    $strClass = $GLOBALS['TL_DCA'][$this->strTable]['list']['label']['label_callback'][0];
                    $strMethod = $GLOBALS['TL_DCA'][$this->strTable]['list']['label']['label_callback'][1];

                    $this->import($strClass);
                    $return .= $this->$strClass->$strMethod($row, $label, $this);
                }
                else
                {
                    $return .= $label;
                }

                // Buttons ($row, $table, $root, $blnCircularReference, $childs, $previous, $next)
                $return .= '</td>'.((\Input::get('act') == 'select') ? '
    <td class="tl_file_list tl_right_nowrap"><input type="checkbox" name="IDS[]" id="ids_'.$row['id'].'" class="tl_tree_checkbox" value="'.$row['id'].'"></td>' : '
    <td class="tl_file_list tl_right_nowrap">'.$this->generateButtons($row, $this->strTable, $this->root).'</td>') . '
  </tr>';
            }

            // Close table
            $return .= '
</table>

</div>';

            // Close form
            if (\Input::get('act') == 'select')
            {
                $return .= '

<div class="tl_formbody_submit" style="text-align:right;">

<div class="tl_submit_container">' . (!$GLOBALS['TL_DCA'][$this->strTable]['config']['notDeletable'] ? '
  <input type="submit" name="delete" id="delete" class="tl_submit" accesskey="d" onclick="return confirm(\''.$GLOBALS['TL_LANG']['MSC']['delAllConfirm'].'\');" value="'.specialchars($GLOBALS['TL_LANG']['MSC']['deleteSelected']).'"> ' : '') . (!$GLOBALS['TL_DCA'][$this->strTable]['config']['notEditable'] ? '
  <input type="submit" name="override" id="override" class="tl_submit" accesskey="v" value="'.specialchars($GLOBALS['TL_LANG']['MSC']['overrideSelected']).'">
  <input type="submit" name="edit" id="edit" class="tl_submit" accesskey="s" value="'.specialchars($GLOBALS['TL_LANG']['MSC']['editSelected']).'"> ' : '') . '
</div>

</div>
</div>
</form>';
            }
        }

        return $return;
    }


    /**
     * Compile buttons from the table configuration array and return them as HTML
     * @param array
     * @param string
     * @param array
     * @param boolean
     * @param array
     * @param int
     * @param int
     * @return string
     */
    protected function generateButtons($arrRow, $strTable, $arrRootIds=array(), $blnCircularReference=false, $arrChildRecordIds=null, $strPrevious=null, $strNext=null)
    {
        if (empty($GLOBALS['TL_DCA'][$strTable]['list']['operations']))
        {
            return '';
        }

        $return = '';

        foreach ($GLOBALS['TL_DCA'][$strTable]['list']['operations'] as $k=>$v)
        {
            $v = is_array($v) ? $v : array($v);
            $label = strlen($v['label'][0]) ? $v['label'][0] : $k;
            $title = sprintf((strlen($v['label'][1]) ? $v['label'][1] : $k), $arrRow['pid']);
            $attributes = strlen($v['attributes']) ? ' ' . ltrim(sprintf($v['attributes'], $arrRow['id'], $arrRow['id'])) : '';

            // Call a custom function instead of using the default button
            if (is_array($v['button_callback']))
            {
                $this->import($v['button_callback'][0]);
                $return .= $this->$v['button_callback'][0]->$v['button_callback'][1]($arrRow, $v['href'], $label, $title, $v['icon'], $attributes, $strTable, $arrRootIds, $arrChildRecordIds, $blnCircularReference, $strPrevious, $strNext);

                continue;
            }

            // Generate all buttons except "move up" and "move down" buttons
            if ($k != 'move' && $v != 'move')
            {
                $return .= '<a href="'.$this->addToUrl($v['href'].'&amp;id='.$arrRow['id']).'" title="'.specialchars($title).'"'.$attributes.'>'.\Image::getHtml($v['icon'], $label).'</a> ';
                continue;
            }

            $arrDirections = array('up', 'down');
            $arrRootIds = is_array($arrRootIds) ? $arrRootIds : array($arrRootIds);

            foreach ($arrDirections as $dir)
            {
                $label = strlen($GLOBALS['TL_LANG'][$strTable][$dir][0]) ? $GLOBALS['TL_LANG'][$strTable][$dir][0] : $dir;
                $title = strlen($GLOBALS['TL_LANG'][$strTable][$dir][1]) ? $GLOBALS['TL_LANG'][$strTable][$dir][1] : $dir;

                $label = \Image::getHtml($dir.'.gif', $label);
                $href = strlen($v['href']) ? $v['href'] : '&amp;act=move';

                if ($dir == 'up')
                {
                    $return .= ((is_numeric($strPrevious) && (!in_array($arrRow['id'], $arrRootIds) || empty($GLOBALS['TL_DCA'][$strTable]['list']['sorting']['root']))) ? '<a href="'.$this->addToUrl($href.'&amp;id='.$arrRow['id']).'&amp;sid='.intval($strPrevious).'" title="'.specialchars($title).'"'.$attributes.'>'.$label.'</a> ' : \Image::getHtml('up_.gif')).' ';
                    continue;
                }

                $return .= ((is_numeric($strNext) && (!in_array($arrRow['id'], $arrRootIds) || empty($GLOBALS['TL_DCA'][$strTable]['list']['sorting']['root']))) ? '<a href="'.$this->addToUrl($href.'&amp;id='.$arrRow['id']).'&amp;sid='.intval($strNext).'" title="'.specialchars($title).'"'.$attributes.'>'.$label.'</a> ' : \Image::getHtml('down_.gif')).' ';
            }
        }

        return trim($return);
    }
}
